/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __houdini_SurfaceAccessibleHapi_h__
#define __houdini_SurfaceAccessibleHapi_h__

namespace fe
{

/**************************************************************************//**
    @brief Houdini Engine Surface Binding

	@ingroup houdini
*//***************************************************************************/
class FE_DL_EXPORT SurfaceAccessibleHapi:
	public SurfaceAccessibleBase
{
	public:
								SurfaceAccessibleHapi(void);
virtual							~SurfaceAccessibleHapi(void);

								using SurfaceAccessibleBase::load;

								//* as SurfaceAccessibleI
virtual	BWORD					isBound(void);

virtual	void					attributeSpecs(
										Array<SurfaceAccessibleI::Spec>&
										a_rSpecs,
										SurfaceAccessibleI::Element
										a_element) const;

virtual sp<SurfaceAccessorI>	accessor(SurfaceAccessibleI::Element a_element,
										String a_name,
										SurfaceAccessibleI::Creation a_create);
virtual sp<SurfaceAccessorI>	accessor(SurfaceAccessibleI::Element a_element,
										SurfaceAccessibleI::Attribute
										a_attribute,
										SurfaceAccessibleI::Creation a_create);

virtual	BWORD					load(String a_filename,
										sp<Catalog> a_spSettings);

virtual	sp<SurfaceI>			surface(String a_group);

static	String					lastError(HAPI_Session* a_pHapiSession);

	private:
		sp<SurfaceAccessorHapi>	accessor(void);

		std::map<String,String>			m_optionMap;
		Real							m_frame;

		HAPI_Session					m_hapiSession;
		HAPI_NodeId						m_hapiNodeId;

		I32								m_curveParts;
		I32								m_meshParts;

		Array<SurfaceAccessibleI::Spec>	m_pointSpecs;
		Array<SurfaceAccessibleI::Spec>	m_vertexSpecs;
		Array<SurfaceAccessibleI::Spec>	m_primitiveSpecs;
		Array<SurfaceAccessibleI::Spec>	m_detailSpecs;
};

} // namespace

#endif /* __houdini_SurfaceAccessibleHapi_h__ */
