/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "network/network.h"

#define LISTENQ		5
#define SERV_PORT	7876

int main(void)
{
	fe::UnitTest unitTest;

	try
	{
		fe::sp<fe::Socket>		listener(new fe::Socket);
		fe::sp<fe::Socket>		connection(new fe::Socket);
		fe::SockAddr			server_addr;
		fe::SockAddr			client_addr;

		listener->openStream();
		server_addr.setPort(SERV_PORT);
		listener->bind(server_addr);
		listener->listen(LISTENQ);

		connection = listener->accept(client_addr);

		fe::sp<fe::ChunkSpool> spBuffer(new fe::ChunkSpool);

		fe::sp<fe::ChunkSender> spSender(new fe::ChunkSender);

		spSender->start(spBuffer, connection);

		for(int i = 1; i < 1000000; i += 1000)
		//for(int i = 1; i < 100000; i += 1000)
		{
			fe::milliSleep(10);
			fe::sp<fe::Chunk> spChunk;
			spChunk = new fe::Chunk(i);
			feLogGroup("network","%d %d sz %d\n", spBuffer.isValid(), spChunk.isValid(), spChunk->size());
			spBuffer->put(spChunk.abandon());
		}

		spSender->stop();

	}
	catch(fe::Exception &e)
	{
		e.log();
	}

	return unitTest.failures();
}
