/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

//* based on FastNoise by Jordan Peck
//* see ext/geometry/FastNoise/ for original code (including the MIT license)

//* TODO this needs to be done on the outside
int m_seed=1337;
unsigned char m_perm[512];
unsigned char m_perm12[512];
void FastNoise_SetSeed(int seed)
{
	m_seed=seed;

	std::mt19937 gen(seed);

	for (int i=0; i < 256; i++)
		m_perm[i]=i;

	for (int j=0; j < 256; j++)
	{
		std::uniform_int_distribution<> dis(0,256-j);
		int k=dis(gen)+j;
		int l=m_perm[j];
		m_perm[j]=m_perm[j+256]=m_perm[k];
		m_perm[k]=l;
		m_perm12[j]=m_perm12[j+256]=m_perm[j] % 12;
	}
}


constant uchar perm[512]=
	{
	0, 0, ... //* TODO
	};
constant uchar perm12[512]=
	{
	0, 0, ... //* TODO
	};

int FastNoise_FastFloor(float f)
{ return (f>=0)? (int)(f): (int)(f-1); }

float FastNoise_Lerp(float a,float b,float t)
{ return a+t*(b-a); }

float FastNoise_InterpQuinticFunc(float t)
{ return t*t*t*(t*(t*6-15)+10); }

unsigned char FastNoise_Index2D_12(unsigned char offset,int x,int y)
{
	return m_perm12[(x&0xff)+m_perm[(y&0xff)+offset]];
}

unsigned char FastNoise_Index3D_12(unsigned char offset,int x,int y,int z)
{
	return m_perm12[(x&0xff)+m_perm[(y&0xff)+m_perm[(z&0xff)+offset]]];
}

const float GRAD_X[12]={ 1,-1, 1,-1,  1,-1, 1,-1,  0, 0, 0, 0 };
const float GRAD_Y[12]={ 1, 1,-1,-1,  0, 0, 0, 0,  1,-1, 1,-1 };
const float GRAD_Z[12]={ 0, 0, 0, 0,  1, 1,-1,-1,  1, 1,-1,-1 };

float FastNoise_GradCoord2D(unsigned char offset,int x,int y,
	float xd,float yd)
{
	unsigned char lutPos=FastNoise_Index2D_12(offset,x,y);

	return xd*GRAD_X[lutPos]+yd*GRAD_Y[lutPos];
}

float FastNoise_GradCoord3D(unsigned char offset,int x,int y,int z,
	float xd,float yd,float zd)
{
	unsigned char lutPos=FastNoise_Index3D_12(offset,x,y,z);

	return xd*GRAD_X[lutPos]+yd*GRAD_Y[lutPos]+zd*GRAD_Z[lutPos];
}

float FastNoise_SinglePerlin(unsigned char offset,float x,float y)
{
	int x0=FastNoise_FastFloor(x);
	int y0=FastNoise_FastFloor(y);
	int x1=x0+1;
	int y1=y0+1;

	float xs,ys;

	xs=FastNoise_InterpQuinticFunc(x-(float)x0);
	ys=FastNoise_InterpQuinticFunc(y-(float)y0);

	float xd0=x-(float)x0;
	float yd0=y-(float)y0;
	float xd1=xd0-1;
	float yd1=yd0-1;

	float xf0=FastNoise_Lerp(
			FastNoise_GradCoord2D(offset,x0,y0,xd0,yd0),
			FastNoise_GradCoord2D(offset,x1,y0,xd1,yd0),xs);
	float xf1=FastNoise_Lerp(
			FastNoise_GradCoord2D(offset,x0,y1,xd0,yd1),
			FastNoise_GradCoord2D(offset,x1,y1,xd1,yd1),xs);

	return FastNoise_Lerp(xf0,xf1,ys);
}

float FastNoise_SinglePerlin(unsigned char offset,float x,float y,float z)
{
	int x0=FastNoise_FastFloor(x);
	int y0=FastNoise_FastFloor(y);
	int z0=FastNoise_FastFloor(z);
	int x1=x0+1;
	int y1=y0+1;
	int z1=z0+1;

	float xs=FastNoise_InterpQuinticFunc(x-(float)x0);
	float ys=FastNoise_InterpQuinticFunc(y-(float)y0);
	float zs=FastNoise_InterpQuinticFunc(z-(float)z0);

	float xd0=x-(float)x0;
	float yd0=y-(float)y0;
	float zd0=z-(float)z0;
	float xd1=xd0-1;
	float yd1=yd0-1;
	float zd1=zd0-1;

	float xf00=FastNoise_Lerp(
			astNoise_GradCoord3D(offset,x0,y0,z0,xd0,yd0,zd0),
			FastNoise_GradCoord3D(offset,x1,y0,z0,xd1,yd0,zd0),xs);
	float xf10=FastNoise_Lerp(
			astNoise_GradCoord3D(offset,x0,y1,z0,xd0,yd1,zd0),
			FastNoise_GradCoord3D(offset,x1,y1,z0,xd1,yd1,zd0),xs);
	float xf01=FastNoise_Lerp(
			astNoise_GradCoord3D(offset,x0,y0,z1,xd0,yd0,zd1),
			FastNoise_GradCoord3D(offset,x1,y0,z1,xd1,yd0,zd1),xs);
	float xf11=FastNoise_Lerp(
			astNoise_GradCoord3D(offset,x0,y1,z1,xd0,yd1,zd1),
			FastNoise_GradCoord3D(offset,x1,y1,z1,xd1,yd1,zd1),xs);

	float yf0=FastNoise_Lerp(xf00,xf10,ys);
	float yf1=FastNoise_Lerp(xf01,xf11,ys);

	return FastNoise_Lerp(yf0,yf1,zs);
}

float FastNoise_GetPerlin(float x,float y) const
{
	return SinglePerlin(0,x*0.666,y*0.666);
}

float FastNoise_GetPerlin(float x,float y,float z) const
{
	return SinglePerlin(0,x*0.666,y*0.666,z*0.666);
}

