/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __usd_SurfaceAccessorUsdGraph_h__
#define __usd_SurfaceAccessorUsdGraph_h__

namespace fe
{

/**************************************************************************//**
    @brief USD Graph Access

	@ingroup usd
*//***************************************************************************/
class FE_DL_EXPORT SurfaceAccessorUsdGraph:
	public SurfaceAccessorBase
{
	public:
						SurfaceAccessorUsdGraph(void)
						{	setName("SurfaceAccessorUsdGraph"); }
virtual					~SurfaceAccessorUsdGraph(void)						{}

						using SurfaceAccessorBase::set;
						using SurfaceAccessorBase::spatialVector;

						//* as SurfaceAccessorI
		BWORD			bind(SurfaceAccessibleI::Element a_element,
								SurfaceAccessibleI::Attribute a_attribute)
						{
							if(a_element==SurfaceAccessibleI::e_point &&
									a_attribute==SurfaceAccessibleI::e_position)
							{
								return bindInternal(a_element,"P");
							}
							if(a_element==SurfaceAccessibleI::e_primitive &&
									a_attribute==SurfaceAccessibleI::e_vertices)
							{
								return bindInternal(a_element,"vertices");
							}
							if(a_element==SurfaceAccessibleI::e_primitive &&
									a_attribute==
									SurfaceAccessibleI::e_properties)
							{
								return bindInternal(a_element,"properties");
							}
							return FALSE;
						}
		BWORD			bind(SurfaceAccessibleI::Element a_element,
								const String& a_name)
						{
							m_attribute=SurfaceAccessibleI::e_generic;
							return bindInternal(a_element,a_name);
						}
virtual	U32				count(void) const
						{
							if(!isBound())
							{
								return 0;
							}

							return elementCount(m_element);
						}
virtual	U32				subCount(U32 index) const
						{
							if(m_element==SurfaceAccessibleI::e_primitive &&
									index<m_pNodeArray->size())
							{
								return 10;
							}
							return 0;
						}

virtual	String			string(U32 a_index,U32 a_subIndex=0)
						{
							if(!isBound() || a_subIndex>=m_pNodeArray->size())
							{
								return String();
							}

							if(m_attrName=="type")
							{
								return (*m_pNodeArray)[a_index]->usdType();
							}

							if(m_attrName=="name")
							{
								return (*m_pNodeArray)[a_index]->usdPath();
							}

							if(m_attrName=="parent")
							{
								return (*m_pNodeArray)[a_index]->usdParent();
							}

							return String();
						}
virtual	I32				integer(U32 a_index,U32 a_subIndex=0)
						{
							if(m_attrName=="properties")
							{
								if(a_subIndex==SurfaceAccessibleI::e_openCurve)
								{
									return 1;
								}

								return 0;
							}

							if(m_attrName=="vertices")
							{
								if(a_subIndex<10)
								{
									return 8*a_index+offsetOfVertex(a_subIndex);
								}
							}

							return 0;
						}
virtual	Real			real(U32 a_index,U32 a_subIndex=0)
						{	return 0.0; }
virtual	SpatialVector	spatialVector(U32 a_index,U32 a_subIndex=0)
						{
							if(!isBound())
							{
								return SpatialVector(0.0,0.0,0.0);
							}

							if(m_attrName=="P")
							{
								const I32 nodeIndex=a_index/8;
								if(nodeIndex<m_pNodeArray->size())
								{
									return extentCorner(nodeIndex,
											a_index-nodeIndex*8);
								}
							}

							if(a_index>=m_pNodeArray->size())
							{
								return SpatialVector(0.0,0.0,0.0);
							}

							if(m_attrName=="vertices")
							{
								return extentCorner(a_index,
										offsetOfVertex(a_subIndex));
							}

							if(m_attrName=="extentMin")
							{
								return (*m_pNodeArray)[a_index]->extentMin();
							}

							if(m_attrName=="extentMax")
							{
								return (*m_pNodeArray)[a_index]->extentMax();
							}

							return SpatialVector(0.0,0.0,0.0);
						}

	private:
		const Array< sp<SurfaceAccessibleUsd::PrimNode> >*	m_pNodeArray;

		I32				offsetOfVertex(I32 a_subIndex)
						{
							const I32 table[10]=
									{ 0,1,2,3,0, 4,5,6,7,4 };
							return table[a_subIndex];
						}

		BWORD			isBound(void) const
						{	return TRUE; }

		I32				elementCount(
								SurfaceAccessibleI::Element a_element) const;

		SpatialVector	extentCorner(I32 a_primitiveIndex,I32 a_offset);

virtual	BWORD			bindInternal(SurfaceAccessibleI::Element a_element,
								const String& a_name);
};

} // namespace

#endif /* __usd_SurfaceAccessorUsdGraph_h__ */
