/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __vegetation_Plant_h__
#define __vegetation_Plant_h__

FE_ATTRIBUTE("veg:plantmodel",	"Physical vegetation sp<Component>");
FE_ATTRIBUTE("veg:plantsketch",	"Drawable SketchI in sp<Component>");
FE_ATTRIBUTE("veg:plantname",	"Growth data lookup key");

namespace fe
{

/**************************************************************************//**
	@brief Plant RecordView

	@ingroup vegetation
*//***************************************************************************/
class FE_DL_EXPORT Plant: public RecordView
{
	public:
		Functor< sp<Component> >	plantModel;
		Functor< sp<Component> >	plantSketch;
		Functor<String>				plantName;
		Functor<SpatialVector>		location;	//* collective displacement
		Functor<SpatialVector>		offset;		//* start of separate root
		Functor<Real>				reactivity;
		Functor<Real>				threshold;

				Plant(void)			{ setName("Plant"); }
virtual	void	addFunctors(void)
				{
					add(plantModel,		FE_USE("veg:plantmodel"));
					add(plantSketch,	FE_USE("veg:plantsketch"));
					add(plantName,		FE_USE("veg:plantname"));
					add(location,		FE_USE("spc:at"));
					add(offset,			FE_USE("spc:offset"));
					add(reactivity,		FE_USE("veg:reactivity"));
					add(threshold,		FE_USE("spc:threshold"));
				}
virtual	void	initializeRecord(void)
				{
					plantModel.attribute()->setSerialize(FALSE);
					plantSketch.attribute()->setSerialize(FALSE);

					set(location());
					set(offset());

					reactivity()=0.0;
					threshold()=0.0;
				}
};

} /* namespace */

#endif /* __vegetation_Plant_h__ */

