/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __vegetation_OperatorTree_h__
#define __vegetation_OperatorTree_h__

namespace fe
{

/**************************************************************************//**
	@brief Handler to Sim and Draw a Tree

	@ingroup vegetation
*//***************************************************************************/
class FE_DL_EXPORT TreeOp:
	public OperatorSurfaceCommon,
	public Initialize<TreeOp>
{
	public:

									TreeOp(void);
virtual								~TreeOp(void);

		void						initialize(void);

									//* As HandlerI
virtual	void						handle(Record& a_rSignal);

	private:

		SpatialVector				convertY(SpatialVector a_vector);

		SpatialVector				unconvertY(SpatialVector a_vector);

		BWORD						rebuild(sp<SurfaceAccessibleI>&
											a_rspInputAccessible);
		BWORD						updateWind(sp<SurfaceAccessibleI>&
											a_rspInputAccessible);
		void						replace(sp<SurfaceAccessibleI>&
											a_rspOutputAccessible);
		void						regenerate(void);

		BWORD						m_yUp;
		Real						m_lastFrame;
		I32							m_lastPointCount;
		I32							m_lastPrimitiveCount;

		Real						m_time;
		String						m_treeName;

		Forest						m_forestRV;
		Plant						m_plantRV;
		sp<Scope>					m_spScope;
		sp<SignalerI>				m_spSignalerI;

		Array< std::map<U32,U32> >	m_plantPointMap;
		Array<SpatialVector>		m_plantOffset;

		Array< sp<HandlerI> >		m_modifierArray;
};

} /* namespace */

#endif /* __vegetation_OperatorTree_h__ */

