# this forge file is for site specific tweaks
import os
import sys

forge = sys.modules["forge"]

def local_setup():
	# FE_EXT environment variable will override this list
	# prerequsite modules
	forge.extensions = "autoload:datatool:draw:geometry:image:math:opengl:operate:shape:surface:thread:window"
	# application targets
	forge.extensions += ":meta:houdini:maya:terminal"
	# operators
	forge.extensions += ":grass:ironworks:operator:vegetation"
	# regex and threading
	forge.extensions += ":boostregex:boostthread:pcre:tbb"
	# file formats
	forge.extensions += ":alembic:fbx:json:lua:oiio:openil:ptex:tiff:usd:vdb:xgen:xml"
	# shaders
#	forge.extensions += ":arnold:shade:shader"
	# viewer
#	forge.extensions += ":native:network:opensubdiv:oplab:viewer"
	# other
#	forge.extensions += ":graphviz:opencl:optest:ray"

	# allow OpenMP
	# OpenMP uses up "static TLS" (if you run out, consider setting this False)
	forge.omp = True

	# parent path for the various versions of Houdini
	# (overriden if the HT environment variable is found)
	forge.houdini_installs = '/opt/'

	# default paths for FE Houdini nodes
	forge.houdini_url_root = 'http://freeelectron.org/op/'
	forge.houdini_icon_root = 'http://freeelectron.org/svg/'

	# desired potential Houdini versions
	# a single entry of 'auto' will build every version found
	forge.houdini_versions = [ 'auto' ]
#	forge.houdini_versions = [
#			'11.0.775',
#			'12.0.581',
#			'12.1.125',
#			'12.5.376' ]

	# mapping from Boost compiler string to actual compiler command
	# the default compiler does not need to be listed
	forge.compilers = {
			'gcc44': 'g++-4.4',
			'gcc46': 'g++-4.6'
#			'gcc48': 'g++-4.9'
	}

	# other Boost versions as [version, lib_suffix, include, lib, compiler]
	# version is like 1_46, usually blank for the distribution default
	# lib_suffix can be something like '-gcc43-mt-1_51', if needed
	# the distribution lib is usually in /usr/lib or /usr/lib64
	forge.boost_sources = []
	if os.path.exists('/usr/include/boost/exception'):
		forge.cppmap['boost'] = '-DFE_BOOST_EXCEPTIONS'
		forge.boost_sources += [ [ '', '', '/usr/include', '/usr/lib/x86_64-linux-gnu', '' ] ]
	forge.boost_sources += [ [ '1_64', '', '/opt/boost1.64.0/include', '/opt/boost1.64.0/lib', '' ] ]
	forge.boost_sources += [ [ '1_55', '', '/opt/boost1.55.0/include', '/opt/boost1.55.0/lib', '' ] ]

	# TBB (see boost_sources)
	forge.tbb_sources = []
	if os.path.exists('/usr/include/tbb/compat/condition_variable'):
		forge.tbb_sources = [ [ '', '', '/usr/include', '/usr/lib/x86_64-linux-gnu', '' ] ]

	# Alembic (see boost_sources)
	forge.alembic_sources = []

	# FBX (see boost_sources)
	forge.fbx_sources = [ [ '2016.1.2', '', '/opt/fbx20161_2/include', '/opt/fbx20161_2/lib/gcc4/x64/release', '' ] ]

	# OpenEXR fallback path (if not adjacent to Alembic)
	forge.openexr_source = ''

	# parent path for the various versions of Maya
	# (overriden if the MAYA_ROOT environment variable is found)
	forge.maya_installs = '/usr/autodesk/'

	# desired potential Maya versions
	# a single entry of 'auto' will build every version found
	forge.maya_versions = [ 'auto' ]
#	forge.maya_versions = [
#			'2012',
#			'2013',
#			'2014',
#			'2015',
#			'2016' ]

	# parent path to "libai.so"
	# leave blank to not use Arnold
	forge.arnold_include = "/opt/Arnold-4.2.8.0/include"
	forge.arnold_lib = forge.arnold_include + "/../bin"

	# parent path to "OpenImageIO/imageio.h" & "libOpenImageIO.so"
	# leave blank to not use OpenImageIO
#	forge.oiio_include = 'ext/oiio/oiio/dist/linux64/include'
	forge.oiio_include = "/usr/include"
	forge.oiio_lib = forge.oiio_include + "/../lib"

	# parent paths to "opensubdiv/version.h" and "libosdCPU.so"
	# leave blank to not use OpenSubdiv
	forge.opensubdiv_include = 'ext/opensubdiv/OpenSubdiv'
	forge.opensubdiv_lib = forge.opensubdiv_include + "/build/lib"

	# parent paths to "CL/cl.h" and "libOpenCL.so"
	# leave blank to not use OpenCL
	forge.opencl_include = "/usr/include"
	forge.opencl_lib = "/usr/lib/x86_64-linux-gnu"

	# parent paths to "pcrecpp.h" and "libpcrecpp.so"
	# leave blank to not use PCRE
	# specify 'auto' to get PCRE from OS install
#	forge.pcre_include = 'auto'
	forge.pcre_include = '/opt/pcre-8.41/install/include'
	forge.pcre_lib = forge.pcre_include + "/../lib"

	# parent paths to "Ptexture.h" and "libPtex.so"
	# leave blank to not use Ptex
	forge.ptex_include = 'ext/ptex/ptex-2.0.41/install/include'
	forge.ptex_lib = forge.ptex_include + "/../lib"

	# parent paths to "openvdb.h" and "libopenvdb.so"
	# leave blank to not use OpenVDB
	# specify 'auto' to get OpenVDB from Houdini
	forge.openvdb_include = 'auto'
	forge.openvdb_lib = forge.openvdb_include

	# parent path to "include" and "lib" for USD
	forge.usd_install = 'ext/usd/USD/install'

	# default order to search for compatible implementations
	# 'omp' specifies direct OpenMP (not an FE dynamic library)
	forge.regex_order = [
			'fexPcre',
			'fexBoostRegex1_64',
			'fexBoostRegex1_55',
			'fexBoostRegex1_51',
			'fexBoostRegex1_46',
			'fexBoostRegex1_37',
			'fexBoostRegex' ]
	forge.thread_order = [
			'fexTbb8000',
			'fexTbb6105',
			'fexTbb6100',
			'fexTbb6002',
			'fexTbb5000',
			'fexTbb',
			'fexBoostThread1_64',
			'fexBoostThread1_55',
			'fexBoostThread1_51',
			'fexBoostThread1_46',
			'fexBoostThread1_37',
			'fexBoostThread',
			'omp'
			]
	forge.mutex_order = forge.thread_order

	# which operator libraries to use
	# 'replace' is a new starting list; leave blank to configure automatically
	forge.houdini_oplibs_replace = []
	forge.maya_oplibs_replace = []
	# 'append' is added to the list, perhaps locally created libraries
	forge.houdini_oplibs_append = []
	forge.maya_oplibs_append = []
